<?php declare(strict_types=1);

namespace Infomaker\ProjectPlugin\Admin\SettingsPage;

use Infomaker\Everyware\Support\Collection;
use Infomaker\Everyware\Support\Str;
use Infomaker\Everyware\Twig\View;
use Infomaker\ProjectPlugin\Admin\Message;
use Infomaker\ProjectPlugin\Admin\MessageBag;
use Infomaker\ProjectPlugin\Handler\JsonImporter;
use Infomaker\ProjectPlugin\Interfaces\SettingsPageInterface;
use Infomaker\ProjectPlugin\Models\SettingsParameter;
use Twig_Error_Loader;
use Twig_Error_Runtime;
use Twig_Error_Syntax;

class SettingsParametersPage implements SettingsPageInterface
{
    protected static $slug = 'settings-parameters';

    protected static $title = 'Settings Parameters';

    protected $textDomain = 'everyware_project_plugin_text_domain';

    /** @var JsonImporter */
    protected $jsonImporter;

    public function __construct()
    {
        if(is_admin()) {
            add_action('wp_ajax_everyware_delete_setting_parameter', [$this, 'ajax_delete_setting_parameter']);
        }
    }

    public function getTabSlug(): string
    {
        return Str::slug(static::$slug);
    }

    public function getTabTitle(): string
    {
        return __(static::$title, $this->textDomain);
    }

    public function onPageLoad(): void
    {
        $this->jsonImporter = new JsonImporter();

        if($this->isUpdated()) {
            SettingsParameter::collection()->setCollection($this->getSubmittedParameters())->save();
        }
    }

    protected function isUpdated(): bool
    {
        return isset($_POST['everyware-settings-parameters-submit']);
    }

    protected function getSubmittedParameters(): Collection
    {
        $submittedParameters = Collection::make($_POST)->get('field', []);

        $validParameters = [];
        foreach($submittedParameters as $field => $values) {
            foreach(array_filter($values) as $index => $value) {
                if(!isset($validParameters[$index])) {
                    $validParameters[$index] = [];
                }

                $validParameters[$index][$field] = $value;
            }
        }

        return SettingsParameter::makeCollection($validParameters);
    }

    public function getStatusMessage(): string
    {
        if($this->isUpdated()) {
            Message::create('Success', 'Updated! Changes have been saved.');
        } elseif($this->jsonImporter->isImported()) {
            Message::create('Success', 'Import done! ');
            Message::create('Info', 'Changes have not yet been saved.');
        }

        return MessageBag::messagesToHtml();
    }

    public function pageContent(): string
    {
        $parameters         = SettingsParameter::all();
        $importedParameters = SettingsParameter::makeCollection($this->jsonImporter->getJson());

        if($this->jsonImporter->isImported()) {
            $parameters = $parameters->merge($importedParameters);
        }

        try {
            return View::generate('@projectPlugin/admin/settings-page/settings-parameters', [
                'form_action' => $_SERVER['REQUEST_URI'],
                'text_domain' => $this->textDomain,
                'template'    => [
                    'key'   => ['label' => 'Key', 'value' => ''],
                    'value' => ['label' => 'Value', 'value' => ''],
                ],
                'parameters'  => $parameters->map(function($parameter) {
                    return [
                        'key'   => ['label' => 'Key', 'value' => $parameter->key],
                        'value' => ['label' => 'Value', 'value' => $parameter->value],
                    ];
                })->toArray(),
                'json_import' => $this->jsonImporter->generate($importedParameters->toJson(), SettingsParameter::all()->toJson()),
            ]);
        } catch(Twig_Error_Loader $e) {
        } catch(Twig_Error_Runtime $e) {
        } catch(Twig_Error_Syntax $e) {
            echo 'Do something'; // TODO: FIX IMPLEMENTATION
        }
    }
}
